/*
 * gpmid.h
 *
 * Copyright (c) 2006 - 2009 Krizalys
 * http://www.krizalys.com/
 */


#ifndef KRIZALYS_GPMID_H
#define KRIZALYS_GPMID_H

#ifdef HAVE_WINDOWS_H
#include <windows.h>
#endif // #ifdef HAVE_WINDOWS_H


#ifdef WIN32
#ifdef BUILDING_GPMID
#define GPMID_ENTRY __declspec(dllexport) __stdcall
#else /* #ifdef BUILDING_GPMID */
#define GPMID_ENTRY __declspec(dllimport) __stdcall
#endif /* #ifdef BUILDING_GPMID */
#else /* #ifdef WIN32 */
#define GPMID_ENTRY
#endif /* #ifdef WIN32 */


/* Structures */
typedef struct {
	unsigned char           ucRed;
	unsigned char           ucGreen;
	unsigned char           ucBlue;
	unsigned char           ucWhite;
} __attribute__((__packed__)) GPColor;

typedef struct {
	unsigned int            uiInstrument;
	unsigned char           ucVolume;
	unsigned char           ucBalance;
	unsigned char           ucChorus;
	unsigned char           ucReverb;
	unsigned char           ucPhaser;
	unsigned char           ucTremolo;
	unsigned char           ucBlank1;
	unsigned char           ucBlank2;
} __attribute__((__packed__)) GPMidiChannel;


#ifdef __cplusplus
extern "C" {
#endif /* #ifdef __cplusplus */

/* Library operations */
/**
 * Initializes the library's internal data. This function must be called once,
 * every time the library is loaded, and must be called before any subsequent
 * calls of other functions provided by the library.
 */
GPMID_ENTRY void GPMID_Init();

/**
 * Free the library's internal data. This function must be called once, every
 * time the library is to be unloaded, and must be called after any previous
 * calls of other functions provided by the library.
 */
GPMID_ENTRY void GPMID_Free();

/**
 * Loads a Guitar Pro file. Only one file can be loaded at a time, but the
 * function can be called multiple times to load several files. Use
 * GPMID_Unload() to unload the current loaded file (if any). The function fails
 * if an I/O or memory error occurs, or if a file is already loaded.
 *
 * @param   [in] aFileName - The name of the file to open.
 * @return  Zero on failure, non-zero otherwise.
 */
GPMID_ENTRY int GPMID_Load(char* aFileName);

/**
 * Unloads the current Guitar Pro file (if any) that was previously loaded by a
 * successful call to GPMID_Load(char* aFileName).
 */
GPMID_ENTRY void GPMID_Unload();

/* Header */
GPMID_ENTRY char* GPMID_GetTitle();
GPMID_ENTRY char* GPMID_GetSubtitle();
GPMID_ENTRY char* GPMID_GetInterpret();
GPMID_ENTRY char* GPMID_GetAlbum();
GPMID_ENTRY char* GPMID_GetAuthor();
GPMID_ENTRY char* GPMID_GetCopyright();
GPMID_ENTRY char* GPMID_GetTabAuthor();
GPMID_ENTRY char* GPMID_GetInstructions();
GPMID_ENTRY unsigned int GPMID_GetNoticeLineCount();
GPMID_ENTRY char* GPMID_GetNoticeLine(unsigned int aLineIdx);
GPMID_ENTRY int GPMID_IsTripletFeelEnabled();
GPMID_ENTRY unsigned int GPMID_GetLyricsTrack();
GPMID_ENTRY char* GPMID_GetLyricsLine(unsigned int aLineIdx);
GPMID_ENTRY unsigned int GPMID_GetTempo();
GPMID_ENTRY signed char GPMID_GetKey();
GPMID_ENTRY int GPMID_GetOctave();
GPMID_ENTRY GPMidiChannel* GPMID_GetMidiChannel(unsigned int aPort, unsigned int aChannel);

/* Measures */
GPMID_ENTRY unsigned int GPMID_GetMeasureCount();
GPMID_ENTRY int GPMID_IsMeasureKeyNumeratorPresent(unsigned int aMeasureIdx);
GPMID_ENTRY int GPMID_IsMeasureKeyDenominatorPresent(unsigned int aMeasureIdx);
GPMID_ENTRY int GPMID_IsMeasureBeginRepeatPresent(unsigned int aMeasureIdx);
GPMID_ENTRY int GPMID_IsMeasureEndRepeatPresent(unsigned int aMeasureIdx);
GPMID_ENTRY int GPMID_IsMeasureMarkerPresent(unsigned int aMeasureIdx);
GPMID_ENTRY int GPMID_IsMeasureTonalityPresent(unsigned int aMeasureIdx);
GPMID_ENTRY int GPMID_IsMeasureDoubleBarPresent(unsigned int aMeasureIdx);
GPMID_ENTRY unsigned char GPMID_GetMeasureKeyNumerator(unsigned int aMeasureIdx);
GPMID_ENTRY unsigned char GPMID_GetMeasureKeyDenominator(unsigned int aMeasureIdx);
GPMID_ENTRY unsigned char GPMID_GetMeasureRepeatCount(unsigned int aMeasureIdx);
GPMID_ENTRY unsigned char GPMID_GetMeasureAltEndingCount(unsigned int aMeasureIdx);
GPMID_ENTRY char* GPMID_GetMeasureMarkerText(unsigned int aMeasureIdx);
GPMID_ENTRY GPColor GPMID_GetMeasureMarkerColor(unsigned int aMeasureIdx);
GPMID_ENTRY signed char GPMID_GetMeasureTonality(unsigned int aMeasureIdx);

/* Tracks */
GPMID_ENTRY unsigned int GPMID_GetTrackCount();
GPMID_ENTRY char* GPMID_GetTrackName(unsigned int aTrackIdx);
GPMID_ENTRY unsigned int GPMID_GetTrackStringCount(unsigned int aTrackIdx);
GPMID_ENTRY unsigned int GPMID_GetTrackTuning(unsigned int aTrackIdx, unsigned int aStringIdx);
GPMID_ENTRY unsigned int GPMID_GetTrackMidiPort(unsigned int aTrackIdx);
GPMID_ENTRY unsigned int GPMID_GetTrackMidiChannel(unsigned int aTrackIdx);
GPMID_ENTRY unsigned int GPMID_GetTrackEffectsMidiChannel(unsigned int aTrackIdx);
GPMID_ENTRY unsigned int GPMID_GetTrackFretCount(unsigned int aTrackIdx);
GPMID_ENTRY unsigned int GPMID_GetTrackCapoHeight(unsigned int aTrackIdx);
GPMID_ENTRY GPColor GPMID_GetTrackColor(unsigned int aTrackIdx);

/**
 * Creates a buffer into memory that holds MIDI data. The content of the buffer
 * is what would be read into a raw MIDI file. To free the memory, use
 * GPMID_DestroyMidiBuffer().
 *
 * @param   aBuffer [out] - The buffer containing a MIDI stream.
 * @param   aSize [out] - The size of the above buffer.
 * @return  Zero on failure, non-zero otherwise.
 */
GPMID_ENTRY int GPMID_CreateMidiBuffer(void** aBuffer, unsigned int* aSize);

/**
 * Destroys a buffer from memory that has been previously created by a
 * successful call to GPMID_CreateMidiBuffer().
 *
 * @param   [in] aBuffer - A buffer returned by GPMID_CreateMidiBuffer().
 */
GPMID_ENTRY void GPMID_DestroyMidiBuffer(void* aBuffer);


#ifdef WIN32
APIENTRY BOOL DllMain(HINSTANCE aInstance, DWORD aReason, LPVOID aReserved);
#endif /* #ifdef WIN32 */

#ifdef __cplusplus
} /* extern "C" */
#endif /* #ifdef __cplusplus */

#endif /* #ifndef KRIZALYS_GPMID_H */
